/*++
  The material contained herein is not a license, either        
  expressly or impliedly, to any intellectual property owned    
  or controlled by any of the authors or developers of this     
  material or to any contribution thereto. The material         
  contained herein is provided on an "AS IS" basis and, to the  
  maximum extent permitted by applicable law, this information  
  is provided AS IS AND WITH ALL FAULTS, and the authors and    
  developers of this material hereby disclaim all other         
  warranties and conditions, either express, implied or         
  statutory, including, but not limited to, any (if any)        
  implied warranties, duties or conditions of merchantability,  
  of fitness for a particular purpose, of accuracy or           
  completeness of responses, of results, of workmanlike         
  effort, of lack of viruses and of lack of negligence, all     
  with regard to this material and any contribution thereto.    
  Designers must not rely on the absence or characteristics of  
  any features or instructions marked "reserved" or             
  "undefined." The Unified EFI Forum, Inc. reserves any         
  features or instructions so marked for future definition and  
  shall have no responsibility whatsoever for conflicts or      
  incompatibilities arising from future changes to them. ALSO,  
  THERE IS NO WARRANTY OR CONDITION OF TITLE, QUIET ENJOYMENT,  
  QUIET POSSESSION, CORRESPONDENCE TO DESCRIPTION OR            
  NON-INFRINGEMENT WITH REGARD TO THE TEST SUITE AND ANY        
  CONTRIBUTION THERETO.                                         
                                                                
  IN NO EVENT WILL ANY AUTHOR OR DEVELOPER OF THIS MATERIAL OR  
  ANY CONTRIBUTION THERETO BE LIABLE TO ANY OTHER PARTY FOR     
  THE COST OF PROCURING SUBSTITUTE GOODS OR SERVICES, LOST      
  PROFITS, LOSS OF USE, LOSS OF DATA, OR ANY INCIDENTAL,        
  CONSEQUENTIAL, DIRECT, INDIRECT, OR SPECIAL DAMAGES WHETHER   
  UNDER CONTRACT, TORT, WARRANTY, OR OTHERWISE, ARISING IN ANY  
  WAY OUT OF THIS OR ANY OTHER AGREEMENT RELATING TO THIS       
  DOCUMENT, WHETHER OR NOT SUCH PARTY HAD ADVANCE NOTICE OF     
  THE POSSIBILITY OF SUCH DAMAGES.                              
                                                                
  Copyright 2006, 2007, 2008, 2009, 2010 Unified EFI, Inc. All  
  Rights Reserved, subject to all existing rights in all        
  matters included within this Test Suite, to which United      
  EFI, Inc. makes no claim of right.                            
                                                                
  Copyright (c) 2010, Phoenix Corporation. All rights reserved.   
   
--*/
/*++

Module Name:

  hashesTest.c

Abstract:
  
  This driver supports platform security service

--*/
#include "HashBBTestMain.h"
#include "HashesTest.h"

//
// Handle for the Security Architectural Protocol instance produced by this driver
//
EFI_GUID gEfiHashesArchProtocolGuid = EFI_HASH_PROTOCOL_GUID;
EFI_GUID gEfiHashesBindingProtocolGuid = EFI_HASH_SERVICE_BINDING_PROTOCOL_GUID;


//
// Worker functions
//

/***********************************************************

***********************************************************/
EFI_STATUS
EFIAPI
HashServiceBindingCreateChildDemo (
	IN EFI_HASH_SERVICE_BINDING_PROTOCOL *_this,
	EFI_HANDLE *ChildHandle,
	EFI_HASH_PROTOCOL **Hash
)
{
	EFI_HASH_SERVICE_BINDING_PROTOCOL *HashSb=NULL;
	EFI_STATUS Status;
	HASH_PRIVATE_DATA *_private = HASH_PRIVATE_DATA_FROM_SERVICE_BINDING_THIS(_this);
		
	my_printf((L"\n  gBS->OpenProtocol(EfiHashesBindingProtocol) ..."));
	//
	// Get the ArpServiceBinding Protocol
	//
	if(_this == NULL)
	{
		my_printf((L"open"));
		#if 1
		Status = gtBS->LocateProtocol(
  				&gEfiHashesBindingProtocolGuid,
  				NULL,
  				&HashSb
				);	
		#else			
		Status = gBS->OpenProtocol (
				*ChildHandle,
				&gEfiHashesBindingProtocolGuid,
				(VOID **)&HashSb,
				_private->ImageHandle,
				NULL,			
				EFI_OPEN_PROTOCOL_GET_PROTOCOL
				);
		#endif		
		if (EFI_ERROR (Status)) {
			my_printf((L" fail(%Xh) !", Status));
			return Status;
		}
	}
	else
	{
		HashSb = _this;
	}
	

	//
	// Initialize a ChildHandle
	//
	*ChildHandle = NULL;
	my_printf((L"\n  HashSb->CreateChild ..."));
	//
	// Create a ChildHandle with the Hash Protocol
	//
	Status = HashSb->CreateChild(HashSb, ChildHandle);
	if (EFI_ERROR (Status)) {
		my_printf((L" fail !"));
		goto ErrorExit;
	}
	my_printf((L"\n  HashSb->CreateChild end."));

	
	my_printf((L"\n  gBS->OpenProtocol(EfiHashesArchProtocol) ..."));
	//
	// Retrieve the Hash Protocol from ChildHandle
	//

		Status = gBS->OpenProtocol (
				*ChildHandle,
				&gEfiHashesArchProtocolGuid,
				(VOID **)Hash,
				_private->ImageHandle,
				*ChildHandle,
				EFI_OPEN_PROTOCOL_BY_DRIVER
				);
		if (EFI_ERROR (Status)) {
			my_printf((L" fail(%Xh) !", Status));
			goto ErrorExit;
		}

	
	my_printf((L"\n  SUCCESS."));
	return EFI_SUCCESS;
	
ErrorExit:
	my_printf((L"\n  Fail !"));
	return Status;
}

EFI_STATUS
EFIAPI
HashServiceBindingDestoryChildDemo (
	IN EFI_HASH_SERVICE_BINDING_PROTOCOL *_this,
	EFI_HANDLE ChildHandle
)
{
	EFI_HASH_SERVICE_BINDING_PROTOCOL *HashSb;
	EFI_STATUS Status;
	HASH_PRIVATE_DATA *_private = HASH_PRIVATE_DATA_FROM_SERVICE_BINDING_THIS(_this);
	
	my_printf((L"\nHashServiceBindingDestoryChildDemo:"));
	my_printf((L"\n  gBS->OpenProtocol(EfiHashesBindingProtocol) ..."));
	//
	// Get the Hash Service Binding Protocol
	//
	if(_this == NULL)
	{	
		Status = gBS->OpenProtocol (
				ChildHandle,
				&gEfiHashesBindingProtocolGuid,
				(VOID **)&HashSb,
				_private->ImageHandle,
				ChildHandle,
				EFI_OPEN_PROTOCOL_GET_PROTOCOL
				);
		if (EFI_ERROR (Status)) {
			my_printf((L" fail(%Xh) !", Status));
			return Status;
		}
	}
	else
	{
		HashSb = _this;
	}	
	my_printf((L"\n  HashSb->DestroyChild ..."));

	//
	// Destroy the ChildHandle with the Hash Protocol
	//
	Status = HashSb->DestroyChild (HashSb, ChildHandle);
	if (EFI_ERROR (Status)) {
		my_printf((L" fail(%Xh) !", Status));
		return Status;
	}

	return EFI_SUCCESS;
}



/*********************************************************


*********************************************************/
void hash_dump(unsigned char buffer[], int length)
{
	int index;
	unsigned char *ptr = buffer;

	if(ptr != NULL)
	{
		for(index=0; index<length; index++)
		{
			my_printf(( L"%02x", *ptr++));
		}
	}
}

EFI_GUID gHashAlgorithmGuids[]={
	EFI_HASH_ALGORITHM_MD5_GUID,
	EFI_HASH_ALGORITHM_SHA1_GUID,
	EFI_HASH_ALGORITHM_SHA224_GUID,
	EFI_HASH_ALGORITHM_SHA256_GUID,
	EFI_HASH_ALGORITHM_SHA384_GUID,
	EFI_HASH_ALGORITHM_SHA512_GUID,
};

EFI_STATUS HashProtocolTestGetSize(EFI_HASH_PROTOCOL *This)
{
	int index;
	UINTN hashSize, result=EFI_UNSUPPORTED;
	unsigned char *ptr;

	if(This == NULL)
	{
		logMsg(gHashBBTestMainAssertionGuid00F,"HashProtocolTestGetSize", "with invalid parameter(This=NULL)", result, 1);
		return EFI_UNSUPPORTED;
	}
	
	for(index=0; index<(sizeof(gHashAlgorithmGuids)/sizeof(EFI_GUID )); index++)
	{
		my_printf((L"\n\nTest No.%d", index));
	//  I. HashSize parameter invalided
		result = This->GetHashSize(This, &gHashAlgorithmGuids[index], (UINTN *)NULL);
		logMsg(gHashBBTestMainAssertionGuid003,"EFI_HASH_PROTOCOL.GetHashSize", "with invalid parameter(HashSize=NULL)", result, EFI_INVALID_PARAMETER);
		if(result != EFI_INVALID_PARAMETER)
		{
			return EFI_UNSUPPORTED;
		}

	//  II.1 HashAlgorithm parameter invalided
		result = This->GetHashSize(This, (EFI_GUID *)NULL, &hashSize);
		logMsg(gHashBBTestMainAssertionGuid004,"EFI_HASH_PROTOCOL.GetHashSize", "with invalid parameter(HashAlgorithm=NULL)", result, EFI_INVALID_PARAMETER);
		if(result != EFI_INVALID_PARAMETER)
		{
			return EFI_UNSUPPORTED;
		}

	//  II.2 HashAlgorithm parameter invalided
		ptr = (char *)&gHashAlgorithmGuids[index];
		*ptr = ~(*ptr);
		result = This->GetHashSize(This, &gHashAlgorithmGuids[index], &hashSize);
		logMsg(gHashBBTestMainAssertionGuid005,"EFI_HASH_PROTOCOL.GetHashSize", "with invalid parameter(HashAlgorithm not exist)", result, EFI_UNSUPPORTED);
		if(result != EFI_UNSUPPORTED)
		{
			return EFI_UNSUPPORTED;
		}	
		*ptr = ~(*ptr);

	//  III. parameters valided

		result = This->GetHashSize(This, &gHashAlgorithmGuids[index], &hashSize);
		logMsg(gHashBBTestMainAssertionGuid006,"EFI_HASH_PROTOCOL.GetHashSize", "with valid parameters", result, EFI_SUCCESS);
		if((result != EFI_SUCCESS)||(hashSize>HASHSIZE_MAXIUM))
		{
			my_printf((L" result(%Xh) hashSize(%Xh) !", result, hashSize));
			return EFI_UNSUPPORTED;
		}		
		

		
	}
	return EFI_SUCCESS;
}

EFI_STATUS HashProtocolTestHash(EFI_HASH_PROTOCOL *This)
{
	int index;
	UINTN hashSize, cnt, result=EFI_UNSUPPORTED;
	BOOLEAN extend=FALSE;
	unsigned char *ptr;
	unsigned char hash[128+1];
	char *message[]={"abc","abcdefghijklmnopqrstuvwxyz"};
	unsigned char *hashStd[]={
		"900150983CD24FB0D6963F7D28E17F72",
		"A9993E364706816ABA3E25717850C26C9CD0D89D",
		"23097D223405D8228642A477BDA255B32AADBCE4BDA0B3F7E36C9DA7",
		"BA7816BF8F01CFEA414140DE5DAE2223B00361A396177A9CB410FF61F20015AD",
		"CB00753F45A35E8BB5A03D699AC65007272C32AB0EDED1631A8B605A43FF5BED8086072BA1E7CC2358BAECA134C825A7",
		"DDAF35A193617ABACC417349AE20413112E6FA4E89A97EA20A9EEEE64B55D39A2192992A274FC1A836BA3C23A3FEEBBD454D4423643CE80E2A9AC94FA54CA49F"
	};
	char hashOutAscii[256+1];

	if(This == NULL)
	{
		logMsg(gHashBBTestMainAssertionGuid00F,"HashProtocolTestHash", "with invalid parameter(This=NULL)", result, 1);
		return EFI_UNSUPPORTED;
	}
	
	for(index=0; index<(sizeof(gHashAlgorithmGuids)/sizeof(EFI_GUID)); index++)
	{	
		extend = 0;
		my_printf((L"\n\nTest No.%d", index));
	//  I.1 message parameter invalided
		result = This->Hash(This, &gHashAlgorithmGuids[index], extend, NULL, 3, hash);
		logMsg(gHashBBTestMainAssertionGuid007,"EFI_HASH_PROTOCOL.Hash", "with invalid parameter(Message=NULL)", result, EFI_INVALID_PARAMETER);
		if(result != EFI_INVALID_PARAMETER)
		{
			return EFI_UNSUPPORTED;
		}

	//  I.2 hash parameter invalided
		result = This->Hash(This, &gHashAlgorithmGuids[index], extend, message[0], 3, NULL);
		logMsg(gHashBBTestMainAssertionGuid008,"EFI_HASH_PROTOCOL.Hash", "with invalid parameter(Hash=NULL)", result, EFI_INVALID_PARAMETER);
		if(result != EFI_INVALID_PARAMETER)
		{
			return EFI_UNSUPPORTED;
		}

	//  II.1 HashAlgorithm parameter invalided
		result = This->Hash(This, NULL, extend, message[0], 3, hash);
		logMsg(gHashBBTestMainAssertionGuid009,"EFI_HASH_PROTOCOL.Hash", "with invalid parameter(HashAlgorithm=NULL)", result, EFI_INVALID_PARAMETER);
		if(result != EFI_INVALID_PARAMETER)
		{
			return EFI_UNSUPPORTED;
		}
	
	//  II.2 HashAlgorithm parameter invalided
		ptr = (char *)&gHashAlgorithmGuids[index];
		*ptr = ~(*ptr);
		result = This->Hash(This, &gHashAlgorithmGuids[index], extend, message[0], 3, hash);
		logMsg(gHashBBTestMainAssertionGuid00A,"EFI_HASH_PROTOCOL.Hash", "with invalid parameter(HashAlgorithm not exist)", result, EFI_UNSUPPORTED);
		if(result != EFI_UNSUPPORTED)
		{
			return EFI_UNSUPPORTED;
		}
		
		*ptr = ~(*ptr);

	//  III extend parameter invalided
		extend = 1;
		result = This->Hash(This, NULL, extend, message[0], 3, hash);
		logMsg(gHashBBTestMainAssertionGuid00B,"EFI_HASH_PROTOCOL.Hash", "with invalid parameter(extend=TRUE)", result, EFI_INVALID_PARAMETER);
		if((result != EFI_INVALID_PARAMETER)&&(result != EFI_SUCCESS))
		{
			return EFI_UNSUPPORTED;
		}	

	
	//  IV. parameters valided with extend=0
		extend = 0;
		result = This->Hash(This, &gHashAlgorithmGuids[index], extend, message[0], 3, hash);
		logMsg(gHashBBTestMainAssertionGuid00C,"EFI_HASH_PROTOCOL.Hash", "valid parameters with extend=0", result, EFI_SUCCESS);
		if(result != EFI_SUCCESS)
		{
			return EFI_UNSUPPORTED;
		}	
		
		result = This->GetHashSize(This, &gHashAlgorithmGuids[index], &hashSize);

		//  translate hash result into ASCII string format
		for(cnt=0; cnt<hashSize; cnt++)
		{
			hashOutAscii[cnt*2 + 0] = HEX2ASCII(hash[cnt]>>4);
			hashOutAscii[cnt*2 + 1] = HEX2ASCII(hash[cnt]&0x0f);
		}
		hashOutAscii[cnt*2 + 0] = '\0';

		//  compare hash output result with standard
		for(cnt=0; cnt<hashSize*2; cnt++)
		{
			if(hashOutAscii[cnt]!=hashStd[index][cnt]) // hash output error !
			{				
				my_printf((L"\n    hashStd[%d]:\n%a", index, hashStd[index]));				
				my_printf((L"\n    hashOutAscii:\n"));
				hash_dump(hash, (int)hashSize);

				my_printf((L"\n    [%d][%d]verfy fail !", index, cnt));

				result = EFI_UNSUPPORTED;				
				break;
			}
		}	
		logMsg(gHashBBTestMainAssertionGuid00D,"EFI_HASH_PROTOCOL.Hash", "verify result with valid parameters extend=0", result, EFI_SUCCESS);				

	//  V. parameters valided with extend=1
		extend = 0;
		result = This->Hash(This, &gHashAlgorithmGuids[index], extend, message[0], 2, hash);
		extend = 1;
		result = This->Hash(This, &gHashAlgorithmGuids[index], extend, &message[0][2], 1, hash);		
		logMsg(gHashBBTestMainAssertionGuid00E,"EFI_HASH_PROTOCOL.Hash", "valid parameters with extend=1", result, EFI_SUCCESS);
		if(result != EFI_SUCCESS)
		{
			return EFI_UNSUPPORTED;
		}	
		
		result = This->GetHashSize(This, &gHashAlgorithmGuids[index], &hashSize);

		//  translate hash result into ASCII string format
		for(cnt=0; cnt<hashSize; cnt++)
		{
			hashOutAscii[cnt*2 + 0] = HEX2ASCII(hash[cnt]>>4);
			hashOutAscii[cnt*2 + 1] = HEX2ASCII(hash[cnt]&0x0f);
		}
		hashOutAscii[cnt*2 + 0] = '\0';

		//  compare hash output result with standard
		for(cnt=0; cnt<hashSize*2; cnt++)
		{
			if(hashOutAscii[cnt]!=hashStd[index][cnt]) // hash output error !
			{				
				my_printf((L"\n    hashStd[%d]:\n%a", index, hashStd[index]));				
				my_printf((L"\n    hashOutAscii:\n"));
				hash_dump(hash, (int)hashSize);

				my_printf((L"\n    [%d][%d]verfy fail !", index, cnt));

				result = EFI_UNSUPPORTED;				
				break;
			}
		}
		logMsg(gHashBBTestMainAssertionGuid00F,"EFI_HASH_PROTOCOL.Hash", "verify result with valid parameters extend=1", result, EFI_SUCCESS);					
	}
	return EFI_SUCCESS;
}


// Initialization Functions
//
#define EFI_SHELL_APP_INIT(ImageHandle, SystemTable) \
  if (EFI_ERROR (LibInitializeShellApplication (ImageHandle, SystemTable)) \
      ) { \
    return EFI_ABORTED; \
  }
extern EFI_STATUS
LibInitializeShellApplication (
  IN EFI_HANDLE                   ImageHandle,
  IN EFI_SYSTEM_TABLE             *SystemTable
  );
  
extern EFI_STATUS
LibUnInitializeStrings (
  VOID
  );

extern VOID
InitializeLib (
  IN EFI_HANDLE           ImageHandle,
  IN EFI_SYSTEM_TABLE     *SystemTable
  );
  
EFI_EVENT              TimerEvent;
/********************************************************************************
 *  Wait for any input from user
 ********************************************************************************/
void SampleTestWaitForAnyInput (void)
{
  UINTN                   NoWait = 1, WaitIndex;
  EFI_EVENT               WaitList[1];
  EFI_STATUS              Status;
  EFI_INPUT_KEY           Key;

  Print (L"\r\nPress any key to continue...");

  //
  // Set 1 second periodic timer
  //
  Status = gtBS->SetTimer (TimerEvent, TimerPeriodic, 10000000);

  //
  // Set up a wait list for a key and the timer
  //
  NoWait = 0;
  WaitList[NoWait++] = gtST->ConIn->WaitForKey;

  //
  // Wait for either
  //
  while (NoWait) {
    Status = gtBS->WaitForEvent (NoWait, WaitList, &WaitIndex);

    switch (WaitIndex) {
      case 0:
        //
        // Read the key
        //
        Status = gtST->ConIn->ReadKeyStroke (gtST->ConIn, &Key);
        if (EFI_SUCCESS == Status)
          NoWait = 0;
        break;

      default:
        break;
    }
  }
  Print (L"\r\n");

  //
  // Done, cancle periodic timer
  //
  Status = gtBS->SetTimer (TimerEvent, TimerCancel, 0);
}

#ifdef _SHELL_APP_
      
EFI_DRIVER_ENTRY_POINT (HashesTestInitialize)

EFI_STATUS
EFIAPI
HashesTestInitialize (
  IN EFI_HANDLE        ImageHandle,
  IN EFI_SYSTEM_TABLE  *SystemTable
  )
/*++

Routine Description:

  Initialize the state information for the Security Architectural Protocol

Arguments:

  ImageHandle of the loaded driver
  Pointer to the System Table

Returns:

  Status

  EFI_SUCCESS           - successful installation of the service
  EFI_OUT_OF_RESOURCES  - cannot allocate protocol data structure
  EFI_DEVICE_ERROR      - cannot create the timer service

--*/
{
  EFI_STATUS  Status;
#ifdef TESTA  
  char hash_message[]="abcdefghijklmnopqrstuvwxyz";  
  char hash_output[64];
  int index;
  UINTN hashSize;
#endif
  
	IN EFI_HASH_SERVICE_BINDING_PROTOCOL *hashServiceBindingProtocol=NULL;
	EFI_HANDLE ChildHandle=NULL;
	EFI_HASH_PROTOCOL *Hash=NULL;
	
	IN EFI_HASH_SERVICE_BINDING_PROTOCOL *hashServiceBindingProtocol1=NULL;
	EFI_HANDLE ChildHandle1=NULL;
	EFI_HASH_PROTOCOL *Hash1=NULL;	 
	
	IN EFI_HASH_SERVICE_BINDING_PROTOCOL *hashServiceBindingProtocol2=NULL;
	EFI_HANDLE ChildHandle2=NULL;
	EFI_HASH_PROTOCOL *Hash2=NULL;	 
		
#ifndef SELF_TEST_ENABLE   
   EFI_SHELL_APP_INIT (ImageHandle, SystemTable);
   LibUnInitializeStrings ();
#endif  
  my_printf((L"\nHashesTest:"));
  my_printf((L"\n  EfiInitializeDriverLib ..."));
  
  EfiInitializeTestLib (ImageHandle, SystemTable);
  //InitializeLib (ImageHandle, SystemTable);
  EfiInitializeDriverLib (ImageHandle, SystemTable);
  
  my_printf((L"\n  CreateEvent(EFI_EVENT_TIMER) ..."));
  Status = gtBS->CreateEvent (EFI_EVENT_TIMER, 0, NULL, NULL, &TimerEvent);
  if (EFI_ERROR(Status)) {
    return Status;
  }

  //
  // Initialize the EFI Driver Library
  //

  my_printf((L"\n  ASSERT_PROTOCOL_ALREADY_INSTALLED ..."));
  //
  // Make sure the Security Architectural Protocol is not already installed in the system
  //
  ASSERT_PROTOCOL_ALREADY_INSTALLED (NULL, &gEfiHashesBindingProtocolGuid);

  //
  // Get the Standard Library Interface
  //
  my_printf((L"\n  gtBS->LocateProtocol(EfiHashesBindingProtocol) ..."));  
  Status = gtBS->LocateProtocol(
  				&gEfiHashesBindingProtocolGuid,
  				NULL,
  				&hashServiceBindingProtocol
				);
  if (EFI_ERROR(Status))
  {
  	my_printf((L" fail(%Xh) !", Status));
    /* return Status;  */
  }

  my_printf((L"\n  HashServiceBindingCreateChildDemo ..."));
  Status = HashServiceBindingCreateChildDemo(hashServiceBindingProtocol, &ChildHandle, &Hash);

#ifdef TESTA

  
  my_printf((L"\n  HashProtocolTestGetSize ..."));
  Status = HashProtocolTestGetSize(Hash);
  my_printf((L"\n  End of HashProtocolTestGetSize."));

  
  my_printf((L"\n  HashProtocolTestHash ..."));
  Status = HashProtocolTestHash(Hash);
  my_printf((L"\n  End of HashProtocolTestHash."));

#endif
  
  my_printf((L"\n  HashServiceBindingDestoryChildDemo ..."));
  Status = HashServiceBindingDestoryChildDemo(hashServiceBindingProtocol, ChildHandle);          
  //ASSERT_EFI_ERROR (Status);

  
//==============================================================================================
  my_printf((L"\n  HashServiceBindingCreateChildDemo1 ..."));
  hashServiceBindingProtocol1 = hashServiceBindingProtocol;
  Status = HashServiceBindingCreateChildDemo(hashServiceBindingProtocol1, &ChildHandle1, &Hash1);

  
#ifdef TESTA    
  index=0;
  my_printf((L"\n  Hash[%d](%a) ...", index, hash_message));
  Hash1->GetHashSize(Hash1, &gHashAlgorithmGuids[index], &hashSize);
  Hash1->Hash(Hash1, &gHashAlgorithmGuids[index], 0, hash_message, 26, hash_output);
  hash_dump(hash_output, (int)hashSize);

#endif
  
  my_printf((L"\n  HashServiceBindingCreateChildDemo2 ..."));
  hashServiceBindingProtocol2 = hashServiceBindingProtocol;
  Status = HashServiceBindingCreateChildDemo(hashServiceBindingProtocol2, &ChildHandle2, &Hash2);

  
#ifdef TESTA   
  index=2;
  my_printf((L"\n  Hash[%d](%a) ...", index, hash_message));
  Hash2->GetHashSize(Hash2, &gHashAlgorithmGuids[index], &hashSize);
  Hash2->Hash(Hash2, &gHashAlgorithmGuids[index], 0, hash_message, 26, hash_output);
  hash_dump(hash_output, (int)hashSize);

#endif   

  my_printf((L"\n  HashServiceBindingDestoryChildDemo1 ..."));
  Status = HashServiceBindingDestoryChildDemo(hashServiceBindingProtocol1, ChildHandle1);  
    
  my_printf((L"\n  HashServiceBindingDestoryChildDemo2 ..."));
  Status = HashServiceBindingDestoryChildDemo(hashServiceBindingProtocol2, ChildHandle2);    
       
//==============================================================================================

  my_printf((L"\n  done!"));

  
  
  Status = gtBS->CloseEvent (TimerEvent);
  if (EFI_ERROR(Status)) {
	my_printf((L"\n  gtBS->CloseEvent %Xh!\n", Status));
  }
  my_printf((L"\n  Bye-bye!\n"));
  Status = EFI_SUCCESS;
  return Status;
}

#endif
