/*++
  The material contained herein is not a license, either        
  expressly or impliedly, to any intellectual property owned    
  or controlled by any of the authors or developers of this     
  material or to any contribution thereto. The material         
  contained herein is provided on an "AS IS" basis and, to the  
  maximum extent permitted by applicable law, this information  
  is provided AS IS AND WITH ALL FAULTS, and the authors and    
  developers of this material hereby disclaim all other         
  warranties and conditions, either express, implied or         
  statutory, including, but not limited to, any (if any)        
  implied warranties, duties or conditions of merchantability,  
  of fitness for a particular purpose, of accuracy or           
  completeness of responses, of results, of workmanlike         
  effort, of lack of viruses and of lack of negligence, all     
  with regard to this material and any contribution thereto.    
  Designers must not rely on the absence or characteristics of  
  any features or instructions marked "reserved" or             
  "undefined." The Unified EFI Forum, Inc. reserves any         
  features or instructions so marked for future definition and  
  shall have no responsibility whatsoever for conflicts or      
  incompatibilities arising from future changes to them. ALSO,  
  THERE IS NO WARRANTY OR CONDITION OF TITLE, QUIET ENJOYMENT,  
  QUIET POSSESSION, CORRESPONDENCE TO DESCRIPTION OR            
  NON-INFRINGEMENT WITH REGARD TO THE TEST SUITE AND ANY        
  CONTRIBUTION THERETO.                                         
                                                                
  IN NO EVENT WILL ANY AUTHOR OR DEVELOPER OF THIS MATERIAL OR  
  ANY CONTRIBUTION THERETO BE LIABLE TO ANY OTHER PARTY FOR     
  THE COST OF PROCURING SUBSTITUTE GOODS OR SERVICES, LOST      
  PROFITS, LOSS OF USE, LOSS OF DATA, OR ANY INCIDENTAL,        
  CONSEQUENTIAL, DIRECT, INDIRECT, OR SPECIAL DAMAGES WHETHER   
  UNDER CONTRACT, TORT, WARRANTY, OR OTHERWISE, ARISING IN ANY  
  WAY OUT OF THIS OR ANY OTHER AGREEMENT RELATING TO THIS       
  DOCUMENT, WHETHER OR NOT SUCH PARTY HAD ADVANCE NOTICE OF     
  THE POSSIBILITY OF SUCH DAMAGES.                              
                                                                
  Copyright 2006, 2007, 2008, 2009, 2010 Unified EFI, Inc. All  
  Rights Reserved, subject to all existing rights in all        
  matters included within this Test Suite, to which United      
  EFI, Inc. makes no claim of right.                            
                                                                
  Copyright (c) 2010, Phoenix Corporation. All rights reserved.   
   
--*/
/*++

Module Name:

  hashesTest.h

Abstract:
  
  This driver supports platform security service

--*/

#ifndef _HASH_STUB_ARCH_PROTOCOL_H
#define _HASH_STUB_ARCH_PROTOCOL_H

#include "hashes.h"


//
// Function prototypes
//

EFI_STATUS
EFIAPI
HashesStubInitialize (
  IN EFI_HANDLE        ImageHandle,
  IN EFI_SYSTEM_TABLE  *SystemTable
  );
  

EFI_FORWARD_DECLARATION (EFI_HASH_SERVICE_BINDING_PROTOCOL);


typedef
EFI_STATUS
(EFIAPI EFI_SERVICE_BINDING_CREATE_CHILD) (
	IN EFI_HASH_SERVICE_BINDING_PROTOCOL *This,
	IN OUT EFI_HANDLE *ChildHandle
);

typedef
EFI_STATUS
(EFIAPI EFI_SERVICE_BINDING_DESTROY_CHILD) (
	IN EFI_HASH_SERVICE_BINDING_PROTOCOL *This,
	IN EFI_HANDLE ChildHandle
);

typedef struct _HASH_PRIVATE_DATA {
	EFI_HASH_PROTOCOL hashServiceProtocol;
	EFI_HANDLE ChildHandle;
	EFI_HANDLE HashInterface;
	EFI_HASH_STATE hashState;
	int NumberCreated;
	int Destroy;
	EFI_HANDLE ImageHandle;
	EFI_HANDLE HashDriverBindingHandle;	
} HASH_PRIVATE_DATA;

typedef struct _EFI_HASH_SERVICE_BINDING_PROTOCOL {
	EFI_SERVICE_BINDING_CREATE_CHILD *CreateChild;
	EFI_SERVICE_BINDING_DESTROY_CHILD *DestroyChild;
	HASH_PRIVATE_DATA PrivateData;
} EFI_HASH_SERVICE_BINDING_PROTOCOL;

#define HASH_PRIVATE_DATA_FROM_SERVICE_BINDING_THIS(x) (&x->PrivateData)
#define HASH_PRIVATE_DATA_FROM_HASH_THIS(x) (HASH_PRIVATE_DATA*)(x)

#define TESTA    



#ifdef DEBUG_HASHTEST
	#define my_printf(x)  Print	
	#define my_logMsgPrintf  Print
#else
	#define my_printf(x) // Print	
	#define my_logMsgPrintf  Print
#endif


#define logMsg(_guid,_title,_message,_result,_maskRes) \
  StandardLib->RecordAssertion (StandardLib,\
                    ( (_maskRes==_result) ? EFI_TEST_ASSERTION_PASSED : EFI_TEST_ASSERTION_FAILED ),\
                   _guid,\
                   L"EFI Hash Service Protocol Testing ",\
                   L" - %a\n  %a\n  %a:%d\n  Status(0x%x) - %r",\
                   _title, _message, \
                   __FILE__, (UINTN)__LINE__,\
                   _result, ( (_maskRes==_result) ? EFI_SUCCESS : EFI_UNSUPPORTED )\
                   );

extern EFI_GUID gEfiHashesArchProtocolGuid;
extern EFI_GUID gEfiHashesBindingProtocolGuid;

EFI_STATUS
EFIAPI
HashServiceBindingCreateChildDemo (
	IN EFI_HASH_SERVICE_BINDING_PROTOCOL *_this,
	EFI_HANDLE *ChildHandle,
	EFI_HASH_PROTOCOL **Hash
);
EFI_STATUS
EFIAPI
HashServiceBindingDestoryChildDemo (
	IN EFI_HASH_SERVICE_BINDING_PROTOCOL *_this,
	EFI_HANDLE ChildHandle
);
EFI_STATUS HashProtocolTestGetSize(EFI_HASH_PROTOCOL *This);
EFI_STATUS HashProtocolTestHash(EFI_HASH_PROTOCOL *This);

#endif
